# ifndef BIBLIOTECASCOMUNES
    # include "bibliotecasComunes.h"
    # define BIBLIOTECASCOMUNES
# endif // BIBLIOTECASCOMUNES


/*
Los siguientes subalgoritmos trabajan con cadenas de caracteres
que representan nmeros enteros.
Presuponen que las cadenas tienen un formato correcto:
estn formadas por caracteres que representan dgitos
('0', '1', '2', '3', '4', '5', '6', '7', '8', '9')
y, en todo caso, un guin inicial para representar un entero negativo.
Son vlidas las cadenas "123", "-12", "00321"
Las siguientes cadenas no son vlidas: "1A" (incluye el carcter 'A') ,
"2-3" (hay un guin que no est al inicio).
En caso de recibir como argumento una cadena no vlida, el valor devuelto es indefinido.

El programador debe asegurarse de que:
* las cadenas pasadas como parmetros dato o dato-resultado
  son vlidas (tener en formato correcto)
* las cadenas pasadas como parmetros dato-resultado o resultado
  tengan la longitud adecuada para contener el resultado.
Por ejemplo, al multiplicar dos cadenas que representan enteros de 5 y 7 cifras,
la cadena resultado debe tener al menos 14 caracteres:
12 para los dgitos del resultado, 1 para el signo eventual y el carcter final '\0'
*/

int compara (const char * n1, const char * n2);
void suma (const char * sumando1, const char * sumando2, char * result);
void resta (const char * minuendo, const char * sustraendo, char * result);
void multiplicacion (const char * f1, const char * f2, char * result);
void division (const char* D, const char* d, char * cociente, char * resto);

/*
int compara (const char * n1, const char * n2);
Descripcin:
    Dadas dos cadenas de caracteres que representan enteros, devuelve un entero
    que indica si el primero es igual, mayor o menor que el segundo.

Parmetros:
    * n1, n2    son punteros a constante. Apuntan a sendos arrays de caracteres
                que representan los dos sumandos.
                A travs de n1 y 12 no se pueden modificar los arrays.


Devuelve:
        Un entero:
            * 0 si ambos enteros son iguales
            * 1 si el entero representado por n1 es mayor que el de n2
            * -1 si el entero representado por n1 es menor que el de n2

Ejemplo:
    void pruebaCompara ()
    {
        char n1[5] = "1234", n2[5] = "123";
        cout << compara (n1, n2) << endl;
        cout << compara (n2, n1) << endl;
        cout << compara (n1, n1) << endl;

        strcpy (n1,"-123"); strcpy (n2, "-12");
        cout << compara (n1, n2) << endl;
        cout << compara (n2, n1) << endl;
        cout << compara (n1, n1) << endl;
    }
    Al ejecutarse, muestra por pantalla:
    1
    -1
    0
    -1
    1
    0
*/
/*
void suma (const char * s1, const char * s2, char * sol);
Descripcin:
    Dadas dos cadenas de caracteres que representan enteros,
    devuelve una cadena que representa su suma.

Parmetros:
    * s1, s2    son punteros a constante. Apuntan a sendos arrays de caracteres
                que representan los dos sumandos.
                A travs de s1 y s2 no se pueden modificar los arrays.
    * sol       es un puntero a un array de caracteres donde se almacenar el resultado.

Requisitos:
    *   s1 y s2 deben tener el formato correcto.
    *   Para evitar desbordamientos, el array al que apunta sol
        debe ser lo suficientemente grande para almacenar el resultado.
    En particular, si n1, n2 y nSol son respectivamente los nmeros de caracteres
    de s1, s2 y sol, debe satisfacerse:
        nSol > mximo (n1, n2)

Ejemplo
    void ejemploSuma()
    {
        char s1[] = "9999",  s2[]="-420";
        int n1 = strlen (s1), n2 = strlen (s2);
        int n12 = (n1>n2)?n1: n2;  //mximo de n1, n2

        char sol1[n1+1], sol2 [n12+1];

        suma (s1, s1, sol1);
        suma (s1, s2, sol2);

        cout << s1 << " + " << s1 << " = " << sol1 << endl;
        cout << s1 << " + " << s2 << " = " << sol2 << endl;
    }

    Al ejecutarse, se muestra por pantalla:
        9999 + 9999 = 19998
        9999 + -420 = 9579
*/


/*
void resta (const char * s1, const char * s2, char * sol);
Descripcin:
    Dadas dos cadenas de caracteres que representan enteros,
    devuelve una cadena que representa su resta.

Parmetros:
    * s1, s2    son punteros a constante. Apuntan a sendos arrays de caracteres
                que representan los dos sumandos.
                A travs de s1 y s2 no se pueden modificar los arrays.
    * sol       es un puntero a un array de caracteres donde se almacenar el resultado.

Requisitos:
    *   s1 y s2 deben tener el formato correcto.
    *   Para evitar desbordamientos, el array al que apunta sol debe ser
        lo suficientemente grande para almacenar el resultado.
        En particular, si n1, n2 y nSol son respectivamente los nmeros de caracteres
        de s1, s2 y sol, debe satisfacerse:
        nSol > mximo (n1, n2)

Ejemplo:
    void ejemploResta()
    {
        char s1[] = "9999", s2[]="-9999";
        int n1 = strlen (s1), n2 = strlen (s2);
        int n12 = (n1>n2)?n1: n2;  //mximo de n1, n2

        char sol1[n1+1], sol2[n12+1];

        resta (s1, s1, sol1);
        cout << s1 << " - " << s1 << " = " << sol1 << endl;

        resta (s1, s2, sol2);
        cout << s1 << " - " << s2 << " = " << sol2 << endl;

        resta (s2, s1, sol2);
        cout << s2 << " - " << s1 << " = " << sol2 << endl;
    }

    Al ejecutarse, se muestra por pantalla:
        9999 - 9999 = 0
        9999 - -9999 = 19998
        -9999 - 9999 = -19998
*/

/*
void multiplicacion (const char * s1, const char * s2, char * sol);
Descripcin:
    Dadas dos cadenas de caracteres que representan enteros,
    devuelve una cadena que representa su producto.

Parmetros:
    * s1, s2    son punteros a constante. Apuntan a sendos arrays de caracteres
                que representan los dos sumandos.
                A travs de s1 y s2 no se pueden modificar los arrays.
    * sol       es un puntero a un array de caracteres donde se almacenar el resultado.

Requisitos:
    *   s1 y s2 deben tener el formato correcto.
    *   Para evitar desbordamientos, el array al que apunta sol debe ser
        lo suficientemente grande para almacenar el resultado.
            En particular, si n1, n2 y nSol son respectivamente
            los nmeros de caracteres de s1, s2 y sol, debe satisfacerse:
            nSol > n1 + n2

Ejemplo:
    void ejemploMultiplicacion()
    {
        char s1[] = "1111", s2[]="-10";
        int n1 = strlen (s1), n2 = strlen (s2);

        char sol1[2*n1+1], sol2[n1+n2+1];

        multiplicacion (s1, s1, sol1);
        cout << s1 << " * " << s1 << " = " << sol1 << endl;

        multiplicacion (s1, s2, sol2);
        cout << s1 << " * " << s2 << " = " << sol2 << endl;
    }

    Al ejecutarse, se muestra por pantalla:
        1111 * 1111 = 1234321
        1111 * -10 = -11110
*/

/*
void division (const char * D, const char * d, char * c, char * r);
Descripcin:
    Dadas dos cadenas de caracteres que representan enteros,
    devuelve dos cadenas que representa, respectivamente,
    su cociente y su resto.

Parmetros:
    * D, d      son punteros a constante. Apuntan a sendos arrays de caracteres
                que representan los dos sumandos.
                A travs de s1 y s2 no se pueden modificar los arrays.
    * c, r:     son punteros a sendos arrays de caracteres donde se almacenarn
                el cociente y el resto, respectivamente.

Requisitos:
    *   D y d deben tener el formato correcto.
    *   Para evitar desbordamientos, el array al que apunta sol
        debe ser lo suficientemente grande para almacenar el resultado.
        En particular, si nD, nd, nC y nR  son respectivamente los nmeros de caracteres
        de D, d, c y r, debe satisfacerse:
            nC >= nD
            nR >= nD

Ejemplo:
    void ejemploDivision()
    {
        char D[] = "1234322", s2[]="111";
        int n1 = strlen (s1), n2 = strlen (s2);

        char c[2*n1+1], sol2[n1+n2+1];

        multiplicacion (s1, s1, sol1);
        cout << s1 << " * " << s1 << " = " << sol1 << endl;

        multiplicacion (s1, s2, sol2);
        cout << s1 << " * " << s2 << " = " << sol2 << endl;
    }

    Al ejecutarse, se muestra por pantalla:
        1111 * 1111 = 1234321
        1111 * -10 = -11110
*/
