#include <math.h>

# ifndef ARITMETICABASICA
    # include "AritmeticaBasica.h"
    # define ARITMETICABASICA
# endif // ARITMETICABASICA
# ifndef ALGORITMOSRSA
    # include "algoritmosRSA.h"
    # define ALGORITMOSRSA
# endif // ALGORITMOSRSA


const char * const listaPrimos[] =
{"7", "11", "13", "17", "19", "23", "29", "31", "37", "41", "43", "47", "51", "53", "59", "61", "67", "71",
 "73", "79", "83", "89", "97"};
const char ultimaCifra[3][4] = {{'1','7','1','7'}, {'1','3','7','9'}, {'3','9','3','9'}};

//Genera un entero aleatorio en el intervalo [0, nMax-1]
int _enteroAleatorio (int nMax)
{
    int aux = (int) (floor) (nMax *rand ()/RAND_MAX);
    if (aux == nMax) aux = nMax -1;
    return aux;
}

// Genera un nmero aleatorio de hasta nD dgitos
char * _generaNumero (int nD, char * candidato)
{
   int i;
   for (i=0; i< nD; i++)
       candidato [i] = _enteroAleatorio(10) + '0';
   candidato[nD] = '\0';

   return candidato;
}

// Genera un nmero aleatorio de exactamente nD dgitos
// que no es mltiplo de los primeros primos: 2, 3, 5...
char * _generaCandidato (unsigned int nD, char * candidato)
{
   unsigned int i,  sum=0;
   char aux[nD+1], valorMaximo[nD+2], acumula[nD+2], cociente[nD+1], resto[nD+1];
   bool hallado= false;

   valorMaximo[0]='1';
   for (i=1; i<nD+1; i++) valorMaximo[i] ='0';
   valorMaximo[i] = '\0';

   do
   {
            //srand(time(NULL));

           candidato = _generaNumero(nD, candidato);

           //Primer dgito (debe ser distinto de 0
           candidato[0] = _enteroAleatorio (9) + 1 +'0';

           //Aseguro que el nmero no sea mltiplo de 2, de 3 ni de 5
           for (i=0, sum=0; i<nD-1; i++)
             sum = (sum + (candidato[i]-'0'))%3;

           candidato[nD-1] = ultimaCifra [sum][_enteroAleatorio(4)];

           //Aseguro que el nmero no sea mltiplo de los siguientes primeros primos
           strcpy (aux, "30");
           suma (candidato, aux, acumula);
           for (i=0; i< sizeof (listaPrimos)/sizeof (double *) && compara (acumula, valorMaximo)<0; i++)
           {
               division (candidato, listaPrimos[i], cociente, resto);
               if (compara (resto, "0")==0)
                    strcpy (candidato, acumula);
               multiplicacion (aux, listaPrimos[i], aux);
               suma (candidato, aux, acumula);
           }
           hallado = true;
           /*for (i=0; i< sizeof (listaPrimos)/sizeof (double *) && hallado== true; i++)
           {
               division (candidato, listaPrimos[i], cociente, resto);
               if (compara (resto, "0")==0)
                    hallado = false;
           }*/
   }
   while (hallado == false);

   return candidato;
}


// Dado un nmero n, devuelve dos, r y s, t.q.  n = 2^s r con s prima
void _preparaMiller (const char * numero, char * s, char * r)
{
    char resto [strlen (numero)+1], cociente [strlen (numero)+1];


    resta (numero, "1", cociente);
    strcpy (s, "0");

    do
    {
        division (cociente, "2", cociente, resto);
        suma (s, "1", s);
        //printf ("\t_preparaMiller: %s s=%s resto=%s s=%s\n",numero,  cociente, resto,s);
    }
    while ( strcmp (resto, "0") == 0);

    resta (s, "1", s);
    multiplicacion (cociente, "2", r);
    suma (r, "1", r);
}

bool _compruebaMillerRabin (const char * numero, const char *s0, const char *r0)
{
    char  r [strlen (r0)+1], s [strlen (s0)+1], aux[strlen (numero)+1] ;
    strcpy (s, s0);
    strcpy (r, r0);

    strcpy (aux, "1");
    //printf ("%s = 2^%s * %s = ", numero, s, r );
    while ( strcmp (s, "0") != 0)
    {
        multiplicacion (aux, "2", aux);
        resta (s, "1",s);
    }
    //printf (" %s * %s  = ",  aux, r);
    multiplicacion (aux, r, aux);
    resta (numero, aux, aux);
    resta (aux, "1", aux);
    //printf ("\t [%s]\n",aux);

    if (strcmp (aux, "0") == 0) return true;
    else return false;
}

bool _TestMiller (const char * numero, const char *n_1, const char *s0, const char *r, const char * a)
{
    char x [2*strlen (a) + 2], s [strlen (s0)+1], cociente [strlen (numero)+1];
    strcpy (s, s0);

    potenciaModulo (a,r,numero, x);  // calculamos a^r%numero

    if (strcmp (x, "1") == 0 || strcmp (x, n_1) == 0) return true;
    else
    {
        while ( compara (s, "0") > 0)
        {
            //cuadrado (x, x);
            multiplicacion (x,x,x);
            division (x, numero, cociente, x);

            if ( strcmp (x, n_1) == 0) return true;
            else if ( strcmp (x, "1") == 0) return false;
            resta (s, "1",s);
        }
        return false;
    }

}


bool MillerRabin (const char * numero, int k)
{
    char s [strlen (numero)+1], r [strlen (numero)+1], a [strlen (numero)+1], cociente [strlen (numero)+1] ;
    char n_1 [strlen (numero)+1], uno [2];

    strcpy (n_1, numero);
    resta (n_1, "1", n_1);
    strcpy (uno, "1");

    _preparaMiller (numero, s,r);

    /*if (!_compruebaMillerRabin (numero, s, r))
    {
        printf("Fallo en _preparaMiller\n");
        system ("PAUSE");
    }*/


    for (int ik=0; ik < k; ik++)
    {

        _generaNumero (strlen(numero), a);   //Generamos nmero de prueba: a
        //printf ("\t\t MillerRabin: %s[%d] divide a %s [%d] ? \n", a, strlen (a), numero, strlen (numero));
        division (a, numero, cociente, a);  // a < numero
        //printf (" ik = %d  a = %s\n", ik,a);
        //printf ("\t\t MillerRabin: %s divide a %s ? \n\n", a, numero);
        if (! _TestMiller(numero, n_1, s, r, a)) return false;
    }
    return true;


}

int buscaPrimo (int nD, char * primoHallado)
{
    int nIntentos=0;
    bool sigueBuscando = true;


    while (sigueBuscando)
    {
        nIntentos ++;
        _generaCandidato (nD, primoHallado);

        //if (superaCriba (primoHallado, 5))
        {
           //printf ("\t buscaPrimo: %s supera criba\n", primoHallado);
           if ( MillerRabin (primoHallado, 30))
             sigueBuscando = false;
        }
    }

    return nIntentos;
}

